'''Symbol/symbol bigram/word/word bigram frequency analyzer with excel output.'''
from frequency_analysis.frequency import Analysis
from frequency_analysis.results import Result

__version__ = '0.1.0'
